@@ -1,12 +1,12 @@
-#  File R/formula.utilities.R in package statnet.common, part of the Statnet suite
-#  of packages for network analysis, https://statnet.org .
+#  File R/formula.utilities.R in package statnet.common, part of the
+#  Statnet suite of packages for network analysis, https://statnet.org .
 #
 #  This software is distributed under the GPL-3 license.  It is free,
 #  open source, and has the attribution requirements (GPL Section 7) at
-#  https://statnet.org/attribution
+#  https://statnet.org/attribution .
 #
-#  Copyright 2007-2019 Statnet Commons
-#######################################################################
+#  Copyright 2007-2022 Statnet Commons
+################################################################################
 ###################################################################
 ## This file has utilities whose primary purpose is examining or ##
 ## manipulating ERGM formulas.                                   ##
@@ -33,9 +33,11 @@ NULL
 #' @param keep.onesided if the initial formula is one-sided, keep it
 #'   whether to keep it one-sided or whether to make the initial
 #'   formula the new LHS
+#' @param env an environment for the new formula, if `object` is `NULL`
 #' @param \dots Additional arguments. Currently unused.
 #' @return \code{append_rhs.formula} each return an updated formula
-#'   object
+#'   object; if `object` is `NULL` (the default), a one-sided formula
+#'   containing only the terms in `newterms` will be returned.
 #' @examples
 #' 
 #' ## append_rhs.formula
@@ -45,22 +47,32 @@ NULL
 #' (f3 <- append_rhs.formula(~y+x,structure(list(as.name("z")),sign=-1)))
 #' (f4 <- append_rhs.formula(~y,list(as.name("z")),TRUE))
 #' (f5 <- append_rhs.formula(y~x,~z1-z2))
-#' 
+#' (f6 <- append_rhs.formula(NULL,list(as.name("z"))))
+#' (f7 <- append_rhs.formula(NULL,structure(list(as.name("z")),sign=-1)))
+#'
 #' \dontshow{
 #' stopifnot(f1 == (y~x+z1+z2))
 #' stopifnot(f2 == (y~z))
 #' stopifnot(f3 == (y+x~-z))
 #' stopifnot(f4 == (~y+z))
 #' stopifnot(f5 == (y~x+z1-z2))
+#' stopifnot(f6 == (~z))
+#' stopifnot(f7 == (~-z))
 #' }
 #'
 #' @export
-append_rhs.formula<-function(object,newterms,keep.onesided=FALSE){
+append_rhs.formula <- function(object = NULL, newterms, keep.onesided = FALSE, env = environment(object)){
+  if(is.null(object)) keep.onesided <- TRUE
+
   if(inherits(newterms,"formula")) newterms <- list_rhs.formula(newterms)
+
   for(i in seq_along(newterms)){
     newterm <- newterms[[i]]
     termsign <- if(NVL(attr(newterms, "sign")[i], +1)>0) "+" else "-"
-    if(length(object)==3) object[[3L]]<-call(termsign,object[[3L]],newterm)
+    if(length(object)==0){
+      if(termsign == "-") newterm <- call(termsign, newterm)
+      object <- as.formula(call("~", newterm), env = env)
+    }else if(length(object)==3) object[[3L]]<-call(termsign,object[[3L]],newterm)
     else if(keep.onesided) object[[2L]]<-call(termsign,object[[2L]],newterm)
     else object[[3L]]<- if(termsign=="+") newterm else call(termsign,newterm)
   }
@@ -165,9 +177,8 @@ nonsimp_update.formula<-function (object, new, ..., from.new=FALSE){
   new.rhs <- if(length(new)==2) new[[2L]] else new[[3L]]
   
   sub.dot <- function(c, dot){
-    if(is.null(dot)) c # If nothing to substitute with, just return it.
+    if(is.name(c) && c==".")  dot # If it's a dot, return substitute.
     else if(is.call(c)) as.call(c(list(c[[1L]]), lapply(c[-1], sub.dot, dot))) # If it's a call, construct a call consisting of the call and each of the arguments with the substitution performed, recursively.
-    else if(is.name(c) && c==".")  dot # If it's a dot, return substitute.
     else c # If it's anything else, just return it.
   }
   
@@ -188,7 +199,17 @@ nonsimp_update.formula<-function (object, new, ..., from.new=FALSE){
     }else return(c)
   }
   
-  out <- if(length(new)==2) call("~", deparen(sub.dot(new.rhs, old.rhs))) else call("~", deparen(sub.dot(new.lhs, old.lhs)), deparen(sub.dot(new.rhs, old.rhs)))
+
+  # This is using some argument alchemy to handle the situation in
+  # which object is one-sided but new is two sided with a dot in the
+  # LHS. quote(expr=) creates a missing argument object that gets
+  # substituted in place of a dot. The next statement then checks if
+  # the resulting LHS *is* a missing object (as it is when the
+  # arguments are ~a and .~.) removes the LHS if it is.
+  out <- if(length(new)==2) call("~", deparen(sub.dot(new.rhs, old.rhs)))
+         else if(length(object)==2) call("~", deparen(sub.dot(new.lhs, quote(expr=))), deparen(sub.dot(new.rhs, old.rhs)))
+         else call("~", deparen(sub.dot(new.lhs, old.lhs)), deparen(sub.dot(new.rhs, old.rhs)))
+  if(identical(out[[2]], quote(expr=))) out <- out[-2]
 
   #  a new sub-environment for the formula, containing both
   # the variables from the old formula and the new.
@@ -198,9 +219,9 @@ nonsimp_update.formula<-function (object, new, ..., from.new=FALSE){
   }else{
     # Create a sub-environment also containing variables from environment of new.
     e <- new.env(parent=environment(object))
-    
-    if(identical(from.new,TRUE)) from.new <- ls(pos=environment(new)) # If TRUE, copy all of them (dangerous!).
-    
+
+    if(identical(from.new,TRUE)) from.new <- setdiff(ls(pos=environment(new), all.names=TRUE), "...") # If TRUE, copy all of them but the dots (dangerous!).
+
     for(name in from.new)
       assign(name, get(name, pos=environment(new)), pos=e)
   }
@@ -308,3 +329,74 @@ eval_lhs.formula <- function(object){
 
   eval(object[[2L]],envir=environment(object))
 }
+
+#' Make a copy of an environment with just the selected objects.
+#'
+#' @param object An [`environment`] or an object with
+#'   [`environment()`] and `environment()<-` methods.
+#' @param ... Additional arguments, passed on to lower-level methods.
+#'
+#' @param keep A character vector giving names of variables in the
+#'   environment (including its ancestors) to copy over, defaulting to
+#'   dropping all. Variables that cannot be resolved are silently
+#'   ignored.
+#'
+#' @return An object of the same type as `object`, with updated environment.
+#' @export
+trim_env <- function(object, keep=NULL, ...){
+  UseMethod("trim_env")
+}
+
+#' @describeIn trim_env A method for environment objects.
+#' @export
+trim_env.environment <- function(object, keep=NULL, ...){
+  # NB: The parent should be baseenv(), not emptyenv(), because :: and
+  # ::: are defined in baseenv(), so PKG:::NAME calls won't work.
+  e <- new.env(parent=baseenv())
+  for(vn in keep){
+    try(assign(vn, get(vn, envir=object), envir=e), silent=TRUE)
+  }
+  e
+}
+
+#' @describeIn trim_env Default method, for objects such as [`formula`] and [`function`] that have [`environment()`] and `environment()<-` methods.
+#' @export
+trim_env.default <- function(object, keep=NULL, ...){
+  environment(object) <- trim_env(environment(object), keep, ...)
+  object
+}
+
+#' Replace an object's environment with a simple, static environment.
+#'
+#' @param object An object with the `environment()<-` method.
+#'
+#' @return An object of the same type as `object`, with updated environment.
+#'
+#' @examples
+#' f <- y~x
+#' environment(f) # GlobalEnv
+#'
+#' environment(empty_env(f)) # EmptyEnv
+#'
+#' \dontshow{
+#' stopifnot(identical(environment(empty_env(f)), emptyenv()))
+#' }
+#' @export
+empty_env <- function(object){
+  environment(object) <- emptyenv()
+  object
+}
+
+#' @rdname empty_env
+#' @examples
+#'
+#' environment(base_env(f)) # base package environment
+#'
+#' \dontshow{
+#' stopifnot(identical(environment(base_env(f)), baseenv()))
+#' }
+#' @export
+base_env <- function(object){
+  environment(object) <- baseenv()
+  object
+}
